/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Game;

import Main.*;
import java.util.*;

public class TetrisGame
{
    private GameTicker      m_objTicker;
    private Tetris          m_objApplication;
    private ScoresTable     m_objScoresTable;
    private Sounder         m_objSounder;
    private SettingsKeeper  m_objSettingsKeeper;
    private int             m_nArea[][];
    private int             m_nPreviewArea[][];
    private int             m_nFigureX;
    private int             m_nFigureY;
    private int             m_nFigure;
    private int             m_nFigureBricks[][];
    private int             m_nNextFigure;
    private int             m_nNextFigureBricks[][];

    private int         m_nPosition;
    private int         m_nLeaderScores;
    private int         m_nNextPositionScores;
    private int         m_nPreviousPositionScores;
    private int         m_nScores;
    private int         m_nLines;
    private int         m_nTicksInterval;
    private Random      m_objRandom;
    private int         m_nState;
    
    public static final int     WIDTH   = 10;
    public static final int     HEIGHT  = 20;
    
    public static final int     NOBRICK = 0;
    public static final int     BRICK   = 1;
    public static final int     FIGURE  = 2;
    
    private static final int    FIGURES_COUNT   = 19;
    
    public static final int STATE_NOTSTARTED    = 0;
    public static final int STATE_ACTIVE        = 1;
    public static final int STATE_GAMEOVER      = 2;
    public static final int STATE_PAUSED        = 3;
    
    public TetrisGame(Tetris objApplication, SettingsKeeper sk)
    {
        m_objApplication = objApplication;
        m_objSettingsKeeper = sk;

        m_nArea = new int[WIDTH][HEIGHT];
        m_nPreviewArea = new int[WIDTH][HEIGHT];
        m_objTicker = new GameTicker(this);
        m_nState = STATE_NOTSTARTED;

        m_objSounder = new Sounder();
        m_objSounder.setMute(sk.getMute());
        m_objSounder.setVolume(sk.getVolume());
    }
    
    private void calcAreaPreview()
    {
        int x;
        int y;
        
        cleanPreviewArea();
        for (y = m_nFigureY; y < HEIGHT; y++)
        {
            if (!canLocateFigure(m_nFigureX, y, m_nFigureBricks))
            {
                pasteFigureToAreaPreview(m_nFigureX, y - 1, m_nFigureBricks);
                break;
            }
        }
    }
    
    private boolean canLocateFigure(int     nFigureX,
                                    int     nFigureY,
                                    int     nFigureBricks[][])
    {
        int     nCurrentX;
        int     nCurrentY;
        int     nAreaX;
        int     nAreaY;
        
        for (nCurrentX = 0; nCurrentX < 4; nCurrentX++)
        {
            for (nCurrentY = 0; nCurrentY < 4; nCurrentY++)
            {
                if (nFigureBricks[nCurrentX][nCurrentY] != NOBRICK)
                {
                    nAreaX = nFigureX + nCurrentX;
                    nAreaY = nFigureY + nCurrentY;
                    if (nAreaX < 0 || nAreaX >= WIDTH || nAreaY < 0 || nAreaY >= HEIGHT)
                        return false;
                    if (m_nArea[nAreaX][nAreaY] == BRICK)
                        return false;
                }
            }
        }
        return true;
    }
    
    private void cleanArea()
    {
        int x, y;
        
        for (x = 0; x < WIDTH; x++)
        {
            for (y = 0; y < HEIGHT; y++)
            {
                m_nArea[x][y] = NOBRICK;
                m_nPreviewArea[x][y] = 0;
            }
        }
    }
    
    private void cleanPreviewArea()
    {
        int x, y;
        
        for (x = 0; x < WIDTH; x++)
            for (y = 0; y < HEIGHT; y++)
                m_nPreviewArea[x][y] = 0;
    }
    
    private boolean cutFigureFromArea(int   x,
                                      int   y,
                                      int   nFigureBricks[][])
    {
        int     nCurrentX;
        int     nCurrentY;
        
        if (!canLocateFigure(x, y, nFigureBricks))
            return false;
        
        for (nCurrentX = 0; nCurrentX < 4; nCurrentX++)
            for (nCurrentY = 0; nCurrentY < 4; nCurrentY++)
                if (nFigureBricks[nCurrentX][nCurrentY] != NOBRICK)
                    m_nArea[x + nCurrentX][y + nCurrentY] = NOBRICK;
        return true;
    }
    
    private void cutLine(int    nLineY)
    {
        int x, y;
        
        for (x = 0; x < WIDTH; x++)
        {
            for (y = nLineY; --y >= 0;)
                m_nArea[x][y + 1] = m_nArea[x][y];
            m_nArea[x][0] = NOBRICK;
        }
    }
    
    private int cutLines()
    {
        int x, y;
        int nLinesCutten = 0;
        
        for (y = HEIGHT; --y >= 0;)
        {
            for (x = WIDTH; --x >= 0;)
                if (m_nArea[x][y] != BRICK)
                    break;
            if (x == -1)
            {
                cutLine(y);
                nLinesCutten++;
                y++;
            }
        }
        return nLinesCutten;
    }

    public synchronized void dropFigure()
    {
        if (m_nState == STATE_ACTIVE)
        {
            while (canLocateFigure(m_nFigureX, m_nFigureY + 1, m_nFigureBricks))
                tick(true);
            tick(false);
        }
    }
    
    private void end()
    {
        if (m_nState == STATE_ACTIVE)
        {
            m_objTicker.pause();
            reset();
        }
    }
    
    private void gameOver()
    {
        m_nState = STATE_GAMEOVER;
        m_objTicker.pause();
        m_objSounder.onGameOver();
    }
    
    private void generateNextFigure()
    {
        int     nRandom;
        String  strRandom;
        
        nRandom = m_objRandom.nextInt();
        if (nRandom + 1 < 0)
            nRandom = -nRandom - 1;
        m_nNextFigure = nRandom / (Integer.MAX_VALUE / FIGURES_COUNT + 1);
        m_nNextFigureBricks = getFigureBricks(m_nNextFigure);
    }

    public synchronized int getBrick(int x,
                                     int y)
    {
        if (x < 0 || x >= WIDTH || y < 0 || y >= HEIGHT)
            return NOBRICK;
        return m_nArea[x][y];
    }
    
    public synchronized int getBrickPreview(int  x,
                                            int  y)
    {
        if (x < 0 || x >= WIDTH || y < 0 || y >= HEIGHT)
            return 0;
        return m_nPreviewArea[x][y];
    }
    
    private static int[][] getFigureBricks(int  nFigure)
    {
        int     nResult[][] = new int[4][4];
        
        switch (nFigure)
        {
        case 0:
            nResult[1][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[0][2]   = FIGURE;
            nResult[1][2]   = FIGURE;
            break;
        case 1:
            nResult[0][0]   = FIGURE;
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            break;
        case 2:
            nResult[1][0]   = FIGURE;
            nResult[2][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            break;
        case 3:
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            nResult[2][2]   = FIGURE;
            break;
        case 4:
            nResult[0][0]   = FIGURE;
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            break;
        case 5:
            nResult[1][0]   = FIGURE;
            nResult[2][0]   = FIGURE;
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            break;
        case 6:
            nResult[2][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            break;
        case 7:
            nResult[0][0]   = FIGURE;
            nResult[1][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            break;
        case 8:
            nResult[0][0]   = FIGURE;
            nResult[1][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            break;
        case 9:
            nResult[2][0]   = FIGURE;
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            break;
        case 10:
            nResult[1][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            nResult[2][2]   = FIGURE;
            break;
        case 11:
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            nResult[0][2]   = FIGURE;
            break;
        case 12:
            nResult[1][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            break;
        case 13:
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            break;
        case 14:
            nResult[1][0]   = FIGURE;
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            break;
        case 15:
            nResult[1][0]   = FIGURE;
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            break;
        case 16:
            nResult[1][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[1][2]   = FIGURE;
            nResult[1][3]   = FIGURE;
            break;
        case 17:
            nResult[0][1]   = FIGURE;
            nResult[1][1]   = FIGURE;
            nResult[2][1]   = FIGURE;
            nResult[3][1]   = FIGURE;
            break;
        case 18:
            nResult[0][0]   = FIGURE;
            nResult[0][1]   = FIGURE;
            nResult[1][0]   = FIGURE;
            nResult[1][1]   = FIGURE;
            break;
        }
        return nResult;
    }
    
    public int getLeaderScores()
    {
        return m_nLeaderScores;
    }
    
    public int getLinesCount()
    {
        return m_nLines;
    }
    
    public int getNextPositionScores()
    {
        return m_nNextPositionScores;
    }
    
    public synchronized int getPreviewBrick(int  x,
                                            int  y)
    {
        if (x < 0 || x >= 4 || y < 0 || y >= 4)
            return NOBRICK;
        return m_nNextFigureBricks[x][y];
    }
    
    public int getPreviousPositionScores()
    {
        return m_nPreviousPositionScores;
    }
    
    private int getRotatedFigure(int nFigure)
    {
        switch (nFigure)
        {
        case 0:
            return 1;
        case 1:
            return 2;
        case 2:
            return 3;
        case 3:
            return 0;

        case 4:
            return 5;
        case 5:
            return 4;

        case 6:
            return 7;
        case 7:
            return 6;

        case 8:
            return 9;
        case 9:
            return 10;
        case 10:
            return 11;
        case 11:
            return 8;

        case 12:
            return 13;
        case 13:
            return 14;
        case 14:
            return 15;
        case 15:
            return 12;

        case 16:
            return 17;
        case 17:
            return 16;

        case 18:
            return 18;
        }
        return 18;
    }
    
    public int getScores()
    {
        return m_nScores;
    }
    
    public int getScoresPosition()
    {
        return m_nPosition;
    }
    
    public Sounder getSounder()
    {
        return m_objSounder;
    }
    
    public int getState()
    {
        return m_nState;
    }
    
    public int getTicksInterval()
    {
        return m_nTicksInterval;
    }
    
    private boolean locateNextFigure()
    {
        m_nFigureX = WIDTH / 2;
        m_nFigureY = 0;
        m_nFigure = m_nNextFigure;
        m_nFigureBricks = m_nNextFigureBricks;
        generateNextFigure();
        if (canLocateFigure(m_nFigureX, m_nFigureY, m_nFigureBricks))
        {
            calcAreaPreview();
            pasteFigureToArea(m_nFigureX, m_nFigureY, m_nFigureBricks, FIGURE);
            return true;
        }
        return false;
    }
    
    public synchronized boolean moveFigure(boolean   bMoveLeft)
    {
        int nNewFigureX;
        
        if (m_nState == STATE_ACTIVE)
        {
            nNewFigureX = m_nFigureX + (bMoveLeft ? (-1) : 1);
            if (canLocateFigure(nNewFigureX, m_nFigureY, m_nFigureBricks))
            {
                cutFigureFromArea(m_nFigureX, m_nFigureY, m_nFigureBricks);
                m_nFigureX = nNewFigureX;
                calcAreaPreview();
                pasteFigureToArea(m_nFigureX, m_nFigureY, m_nFigureBricks, FIGURE);
                return true;
            }
        }
        return false;
    }
    
    private boolean pasteFigureToArea(int   x,
                                      int   y,
                                      int   nFigureBricks[][],
                                      int   nBrickOrFigure)
    {
        int     nCurrentX;
        int     nCurrentY;
        
        if (!canLocateFigure(x, y, nFigureBricks))
            return false;
        
        for (nCurrentX = 0; nCurrentX < 4; nCurrentX++)
            for (nCurrentY = 0; nCurrentY < 4; nCurrentY++)
                if (nFigureBricks[nCurrentX][nCurrentY] != NOBRICK)
                    m_nArea[x + nCurrentX][y + nCurrentY] = nBrickOrFigure;
        return true;
    }
    
    private void pasteFigureToAreaPreview(int   x,
                                          int   y,
                                          int   nFigureBricks[][])
    {
        int     nCurrentX;
        int     nCurrentY;
        
        if (!canLocateFigure(x, y, nFigureBricks))
            return;
        
        for (nCurrentX = 0; nCurrentX < 4; nCurrentX++)
            for (nCurrentY = 0; nCurrentY < 4; nCurrentY++)
                if (nFigureBricks[nCurrentX][nCurrentY] != NOBRICK)
                    m_nPreviewArea[x + nCurrentX][y + nCurrentY] = 1;
    }
    
    public synchronized void pause()
    {
        if (m_nState == STATE_ACTIVE)
        {
            m_nState = STATE_PAUSED;
            m_objTicker.pause();
        }
    }
    
    public synchronized void pauseOrResume()
    {
        switch (m_nState)
        {
        case STATE_PAUSED:
            resume();
            break;
        case STATE_ACTIVE:
            pause();
        }
    }
    
    public synchronized void reset()
    {
        cleanArea();
        cleanPreviewArea();
        m_nPosition = 0;
        m_nScores   = 0;
        m_nLines    = 0;
        m_nState    = STATE_NOTSTARTED;
    }
    
    public synchronized void restart()
    {
        reset();
        start();
    }
    
    public synchronized void resume()
    {
        if (m_nState == STATE_PAUSED)
        {
            m_nState = STATE_ACTIVE;
            m_objTicker.resume();
        }
    }
    
    public synchronized boolean rotateFigure()
    {
        if (m_nState == STATE_ACTIVE)
        {
            int nNewFigure = getRotatedFigure(m_nFigure);
            int nNewFigureBricks[][] = getFigureBricks(nNewFigure);

            if (canLocateFigure(m_nFigureX, m_nFigureY, nNewFigureBricks))
            {
                cutFigureFromArea(m_nFigureX, m_nFigureY, m_nFigureBricks);
                m_nFigure = nNewFigure;
                m_nFigureBricks = nNewFigureBricks;
                calcAreaPreview();
                pasteFigureToArea(m_nFigureX, m_nFigureY, m_nFigureBricks, FIGURE);
                return true;
            }
        }
        return false;
    }
    
    public void setScoresTableObject(ScoresTable objScoresTable)
    {
        m_objScoresTable = objScoresTable;
        m_nLeaderScores             = objScoresTable.getLeaderScores();
        m_nNextPositionScores       = objScoresTable.getNextRecordScores(0);
        m_nPreviousPositionScores   = objScoresTable.getPreviousRecordScores(0);
    }
    
    public synchronized void start()
    {
        m_objRandom = new Random();
        generateNextFigure();
        locateNextFigure();
        updateTicksInterval();
        m_nLeaderScores             = m_objScoresTable.getLeaderScores();
        m_nNextPositionScores       = m_objScoresTable.getNextRecordScores(0);
        m_nPreviousPositionScores   = m_objScoresTable.getPreviousRecordScores(0);
        m_objTicker.resume();
        m_nState    = STATE_ACTIVE;
    }
    
    public synchronized boolean tick(boolean bNoCanLocateCheck)
    {
        boolean bResult;
        int     nLinesCutten;
        int     y;
        
        if (bNoCanLocateCheck || canLocateFigure(m_nFigureX, m_nFigureY + 1, m_nFigureBricks))
        {
            cutFigureFromArea(m_nFigureX, m_nFigureY, m_nFigureBricks);
            m_nFigureY++;
            calcAreaPreview();
            pasteFigureToArea(m_nFigureX, m_nFigureY, m_nFigureBricks, FIGURE);
            bResult = true;
        }
        else
        {
            pasteFigureToArea(m_nFigureX, m_nFigureY, m_nFigureBricks, BRICK);
            nLinesCutten = cutLines();
            if (nLinesCutten == 0)
                m_objSounder.onDrop();
            else
            {
                m_nScores   += nLinesCutten * nLinesCutten;
                m_nLines    += nLinesCutten;
                m_nPosition = m_objScoresTable.getScoresPosition(m_nScores);
                m_nNextPositionScores = m_objScoresTable.getNextRecordScores(m_nScores);
                m_nPreviousPositionScores = m_objScoresTable.getPreviousRecordScores(m_nScores);
                m_objSounder.onCutLines();
                //m_objApplication.vibrate();
                updateTicksInterval();
            }
            bResult = locateNextFigure();
            if (!bResult)
                gameOver();
        }
        m_objApplication.repaintView();
        return bResult;
    }
    
    private void updateTicksInterval()
    {
        m_nTicksInterval = 20000 / (m_nLines + 30);
    }
}