/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Interface;

import Main.*;
import javax.microedition.lcdui.*;

public class ColorDialogue extends Canvas implements CommandListener, Localizable
{
    private Tetris              m_objApplication;
    private ColorThemeKeeper    m_objColorThemeKeeper;
    private Displayable         m_objPreviousDisplayable;
    private Localizer           m_objLocalizer;
    private int                 m_nOldColor;
    private int                 m_nColor;
    private boolean             m_bIsColor;
    
    private Command             m_cmdOK;
    private Command             m_cmdCancel;
    private Command             m_cmdReset;

    private Image   m_imgBackground;
    private Font    m_objFont;
    private int     m_nTextColor;
    private int     m_nBackgroundColor;
    private int     m_nBorderColor;
    private int     m_nSelectionColor;
    
    private int     m_nColorBarStartX;
    private int     m_nRColorStartY;
    private int     m_nGColorStartY;
    private int     m_nBColorStartY;
    private int     m_nHColorStartY;
    private int     m_nSColorStartY;
    private int     m_nVColorStartY;
    private int     m_nGrayscaleBarStartX;
    private int     m_nGrayscaleStartY;
    private int     m_nBarHeight;
    private int     m_nBarWidth;
    private int     m_nTextStartX;
    private int     m_nOldSampleStartX;
    private int     m_nNewSampleStartX;
    private int     m_nSampleStartY;
    private int     m_nSampleWidth;
    private int     m_nSampleHeight;

    private int     m_nColorBarsCount;
    private int     m_nCurrentColorBar;
    private int     m_nRColor;
    private int     m_nGColor;
    private int     m_nBColor;
    private int     m_nHColor;
    private int     m_nSColor;
    private int     m_nVColor;
    private int     m_nGrayscale;
    private int     m_nColorParam;
    
    private int     m_nWidth;
    private int     m_nHeight;
    private int     m_nFontLineHeight;
    
    public ColorDialogue(Tetris     objApplication,
                         Localizer  objLocalizer)
    {
        m_objApplication = objApplication;
        m_objLocalizer = objLocalizer;
        m_nWidth = getWidth();
        m_nHeight = getHeight();

        m_objFont = Font.getFont(
            Font.FACE_PROPORTIONAL,
            Font.STYLE_BOLD,
            Font.SIZE_SMALL);

        m_nFontLineHeight = m_objFont.getHeight();
        
        m_cmdOK     = new Command("OK", Command.OK, 0);
        addCommand(m_cmdOK);
        setCommandListener(this);
        localize();
    }
    
    public void commandAction(Command       cmd,
                              Displayable   d)
    {
        if (cmd == m_cmdOK)
        {
            setColor(m_nColor);
            m_objApplication.setColor(m_nColorParam, m_nColor);
            m_objApplication.onBack();
        }
        else if (cmd == m_cmdCancel)
        {
            resetColor();
            m_objApplication.onBack();
        }
        else if (cmd == m_cmdReset)
            resetColor();
    }
    
    private static void drawBarPointer(Graphics g,
                                       int      x,
                                       int      y,
                                       int      nColor)
    {
        g.setColor(nColor);
        g.drawLine(x, y, x, y);
        g.drawLine(x - 1, y - 1, x + 1, y - 1);
        g.drawLine(x - 2, y - 2, x + 2, y - 2);
        g.drawLine(x - 3, y - 3, x + 3, y - 3);
    }
    
    private void keyEvent(int   nKeyCode)
    {
        int nNewColor;
        int nColorDelta         = 0;
        int nNewCurrentColorBar = m_nCurrentColorBar;
        
        switch (nKeyCode)
        {
            case -1:    //  Up
            case KEY_NUM2:
            case UP:
                nNewCurrentColorBar = m_nCurrentColorBar - 1;
                if (nNewCurrentColorBar < 0)
                    nNewCurrentColorBar = m_nColorBarsCount - 1;
                break;
                
            case -2:    //  Down
            case KEY_NUM8:
            case DOWN:
                nNewCurrentColorBar = m_nCurrentColorBar + 1;
                if (nNewCurrentColorBar >= m_nColorBarsCount)
                    nNewCurrentColorBar = 0;
                break;
                
            case -3:    //  Left
            case KEY_NUM4:
            case LEFT:
                nColorDelta = -5;
                break;
                
            case -4:    //  Right
            case KEY_NUM6:
            case RIGHT:
                nColorDelta = 5;
                break;
                
            case -5:    //  Center
            case FIRE:
                commandAction(m_cmdOK, this);
        }
        if (nColorDelta != 0)
        {
            if (!m_bIsColor)
            {
                nNewColor = m_nGrayscale + nColorDelta;
                if (nNewColor < 0)
                    nNewColor = 0;
                if (nNewColor > 255)
                    nNewColor = 255;
                if (nNewColor != m_nGrayscale)
                {
                    m_nGrayscale = nNewColor;
                    updateGrayscale();
                    repaint();
                }
            }
            else
            {
                switch (m_nCurrentColorBar)
                {
                case 0: //  Red
                    nNewColor = m_nRColor + nColorDelta;
                    if (nNewColor < 0)
                        nNewColor = 0;
                    if (nNewColor > 255)
                        nNewColor = 255;
                    if (nNewColor != m_nRColor)
                    {
                        m_nRColor = nNewColor;
                        updateHSV();
                        repaint();
                    }
                    break;
                case 1: //  Green
                    nNewColor = m_nGColor + nColorDelta;
                    if (nNewColor < 0)
                        nNewColor = 0;
                    if (nNewColor > 255)
                        nNewColor = 255;
                    if (nNewColor != m_nGColor)
                    {
                        m_nGColor = nNewColor;
                        updateHSV();
                        repaint();
                    }
                    break;
                case 2: //  Blue
                    nNewColor = m_nBColor + nColorDelta;
                    if (nNewColor < 0)
                        nNewColor = 0;
                    if (nNewColor > 255)
                        nNewColor = 255;
                    if (nNewColor != m_nBColor)
                    {
                        m_nBColor = nNewColor;
                        updateHSV();
                        repaint();
                    }
                    break;
                case 3: //  Hue
                    nNewColor = m_nHColor + nColorDelta;
                    if (nNewColor < 0)
                        nNewColor = 0;
                    if (nNewColor >= 360)
                        nNewColor = 0;
                    if (nNewColor != m_nHColor)
                    {
                        m_nHColor = nNewColor;
                        updateRGB();
                        repaint();
                    }
                    break;
                case 4: //  Saturation
                    nNewColor = m_nSColor + nColorDelta;
                    if (nNewColor < 0)
                        nNewColor = 0;
                    if (nNewColor > 100)
                        nNewColor = 100;
                    if (nNewColor != m_nSColor)
                    {
                        m_nSColor = nNewColor;
                        updateRGB();
                        repaint();
                    }
                    break;
                case 5: //  Value
                    nNewColor = m_nVColor + nColorDelta;
                    if (nNewColor < 0)
                        nNewColor = 0;
                    if (nNewColor > 100)
                        nNewColor = 100;
                    if (nNewColor != m_nVColor)
                    {
                        m_nVColor = nNewColor;
                        updateRGB();
                        repaint();
                    }
                    break;
                }
            }
        }
        if (nNewCurrentColorBar != m_nCurrentColorBar)
        {
            m_nCurrentColorBar = nNewCurrentColorBar;
            repaint();
        }
    }
    
    protected void keyPressed(int nKeyCode)
    {
        keyEvent(nKeyCode);
    }
    
    protected void keyRepeated(int nKeyCode)
    {
        keyEvent(nKeyCode);
    }
    
    public void localize()
    {
        setTitle(m_objApplication.getColorDialogueTitle(m_nColorParam));
        if (m_cmdReset != null)
        {
            removeCommand(m_cmdCancel);
            removeCommand(m_cmdReset);
        }
        m_cmdCancel = new Command(m_objLocalizer.get(Localizer.S_CANCEL), Command.CANCEL, 2);
        m_cmdReset = new Command(m_objLocalizer.get(Localizer.S_RESET), Command.SCREEN, 3);
        addCommand(m_cmdCancel);
        addCommand(m_cmdReset);
    }
    
    protected void paint(Graphics   g)
    {
        int i;
        
        g.drawImage(m_imgBackground, 0, 0, Graphics.LEFT | Graphics.TOP);
        g.setColor(m_nOldColor);
        g.fillRect(
            m_nOldSampleStartX,
            m_nSampleStartY,
            m_nSampleWidth,
            m_nSampleHeight);
        g.setColor(m_nColor);
        g.fillRect(
            m_nNewSampleStartX,
            m_nSampleStartY,
            m_nSampleWidth,
            m_nSampleHeight);
        if (!m_bIsColor)
        {
            drawBarPointer(
                g,
                m_nGrayscaleBarStartX + m_nGrayscale * m_nBarWidth / 255,
                m_nGrayscaleStartY - 2,
                m_nSelectionColor);
            g.setColor(m_nTextColor);
            g.drawString(
                String.valueOf(m_nGrayscale),
                m_nTextStartX,
                m_nGrayscaleStartY,
                Graphics.LEFT | Graphics.TOP);
        }
        else
        {
            for (i = 0; i < 6; i++)
            {
                g.setColor(i == m_nCurrentColorBar ? m_nSelectionColor : m_nBorderColor);
                g.drawRect(
                    m_nColorBarStartX - 1,
                    m_nRColorStartY + (m_nBarHeight + 2 + 6) * i - 1,
                    m_nBarWidth + 2,
                    m_nBarHeight + 2);
            }
            for (i = 0; i <= m_nBarWidth; i++)
            {
                g.setColor(i * 255 / m_nBarWidth, m_nGColor, m_nBColor);
                g.drawLine(
                    m_nColorBarStartX + i, m_nRColorStartY,
                    m_nColorBarStartX + i, m_nRColorStartY + m_nBarHeight);
                g.setColor(m_nRColor, i * 255 / m_nBarWidth, m_nBColor);
                g.drawLine(
                    m_nColorBarStartX + i, m_nGColorStartY,
                    m_nColorBarStartX + i, m_nGColorStartY + m_nBarHeight);
                g.setColor(m_nRColor, m_nGColor, i * 255 / m_nBarWidth);
                g.drawLine(
                    m_nColorBarStartX + i, m_nBColorStartY,
                    m_nColorBarStartX + i, m_nBColorStartY + m_nBarHeight);
                g.setColor(Color.fromHSV(i * 360 / m_nBarWidth, m_nSColor, m_nVColor));
                g.drawLine(
                    m_nColorBarStartX + i, m_nHColorStartY,
                    m_nColorBarStartX + i, m_nHColorStartY + m_nBarHeight);
                g.setColor(Color.fromHSV(m_nHColor, i * 100 / m_nBarWidth, m_nVColor));
                g.drawLine(
                    m_nColorBarStartX + i, m_nSColorStartY,
                    m_nColorBarStartX + i, m_nSColorStartY + m_nBarHeight);
                g.setColor(Color.fromHSV(m_nHColor, m_nSColor, i * 100 / m_nBarWidth));
                g.drawLine(
                    m_nColorBarStartX + i, m_nVColorStartY,
                    m_nColorBarStartX + i, m_nVColorStartY + m_nBarHeight);
            }
            drawBarPointer(
                g,
                m_nRColor * m_nBarWidth / 255 + m_nColorBarStartX,
                m_nRColorStartY - 2,
                m_nCurrentColorBar == 0 ? m_nSelectionColor : m_nBorderColor);
            drawBarPointer(
                g,
                m_nGColor * m_nBarWidth / 255 + m_nColorBarStartX,
                m_nGColorStartY - 2,
                m_nCurrentColorBar == 1 ? m_nSelectionColor : m_nBorderColor);
            drawBarPointer(
                g,
                m_nBColor * m_nBarWidth / 255 + m_nColorBarStartX,
                m_nBColorStartY - 2,
                m_nCurrentColorBar == 2 ? m_nSelectionColor : m_nBorderColor);
            drawBarPointer(
                g,
                m_nHColor * m_nBarWidth / 360 + m_nColorBarStartX,
                m_nHColorStartY - 2,
                m_nCurrentColorBar == 3 ? m_nSelectionColor : m_nBorderColor);
            drawBarPointer(
                g,
                m_nSColor * m_nBarWidth / 100 + m_nColorBarStartX,
                m_nSColorStartY - 2,
                m_nCurrentColorBar == 4 ? m_nSelectionColor : m_nBorderColor);
            drawBarPointer(
                g,
                m_nVColor * m_nBarWidth / 100 + m_nColorBarStartX,
                m_nVColorStartY - 2,
                m_nCurrentColorBar == 5 ? m_nSelectionColor : m_nBorderColor);
            g.setColor(m_nTextColor);
            g.drawString(
                String.valueOf(m_nRColor),
                m_nTextStartX,
                m_nRColorStartY,
                Graphics.LEFT | Graphics.TOP);
            g.drawString(
                String.valueOf(m_nGColor),
                m_nTextStartX,
                m_nGColorStartY,
                Graphics.LEFT | Graphics.TOP);
            g.drawString(
                String.valueOf(m_nBColor),
                m_nTextStartX,
                m_nBColorStartY,
                Graphics.LEFT | Graphics.TOP);
            g.drawString(
                String.valueOf(m_nHColor),
                m_nTextStartX,
                m_nHColorStartY,
                Graphics.LEFT | Graphics.TOP);
            g.drawString(
                String.valueOf(m_nSColor),
                m_nTextStartX,
                m_nSColorStartY,
                Graphics.LEFT | Graphics.TOP);
            g.drawString(
                String.valueOf(m_nVColor),
                m_nTextStartX,
                m_nVColorStartY,
                Graphics.LEFT | Graphics.TOP);
        }
    }
    
    private void resetColor()
    {
        setColor(m_nOldColor);
    }
    
    public void setColor(int    nColor)
    {
        m_nColor = nColor;
        m_nOldColor = nColor;
        m_nRColor   = Color.red(m_nColor);
        m_nGColor   = Color.green(m_nColor);
        m_nBColor   = Color.blue(m_nColor);
        m_nHColor   = Color.hue(m_nColor);
        m_nSColor   = Color.saturation(m_nColor);
        m_nVColor   = Color.value(m_nColor);
        m_nGrayscale = (m_nRColor + m_nGColor + m_nBColor) / 3;
        repaint();
    }
    
    public void setColorParam(int nParam)
    {
        m_nColorParam = nParam;
    }
    
    public void setColorThemeKeeperObject(ColorThemeKeeper objColorThemeKeeper)
    {
        int         i;
        Graphics    g;
        
        m_objColorThemeKeeper   = objColorThemeKeeper;
        m_bIsColor              = m_objColorThemeKeeper.isColor();
        m_nTextColor            = m_objColorThemeKeeper.getTextColor();
        m_nBackgroundColor      = m_objColorThemeKeeper.getBackgroundColor();
        m_nBorderColor          = m_objColorThemeKeeper.getBorderColor();
        m_nSelectionColor       = m_objColorThemeKeeper.getSelectionColor();
        
        m_imgBackground = Image.createImage(m_nWidth, m_nHeight);
        g = m_imgBackground.getGraphics();
        g.drawImage(
            m_objColorThemeKeeper.getBackgroundImage(),
            0, 0,
            Graphics.LEFT | Graphics.TOP);
        
        m_nTextStartX = m_nWidth -
	    m_objFont.charsWidth("255".toCharArray(), 0, 3) -
	    2;
        m_nBarHeight = m_nFontLineHeight / 2;
        m_nOldSampleStartX = 2 + 1;
        m_nSampleWidth = (m_nWidth - m_nOldSampleStartX - 2) / 2 - 2;
        m_nNewSampleStartX = m_nOldSampleStartX + m_nSampleWidth;
        m_nSampleStartY = 2 + 1;
        g.setColor(m_nBorderColor);
        g.setFont(m_objFont);
        if (!m_bIsColor)
        {
            m_nGrayscaleStartY = m_nHeight - m_nBarHeight - 2 - 2 - 6;
            m_nGrayscaleBarStartX = 2 + 1;
            m_nBarWidth = m_nTextStartX - 2 - 4 - 1;
            m_nColorBarsCount = 1;
            g.setColor(m_nSelectionColor);
            g.drawRect(
                m_nGrayscaleBarStartX - 1, m_nGrayscaleStartY - 1,
                m_nBarWidth + 2, m_nBarHeight + 2);
            g.setColor(m_nTextColor);
            for (i = 0; i <= m_nBarWidth; i++)
            {
                g.setGrayScale(i * 255 / m_nBarWidth);
                g.drawLine(
                    i + m_nGrayscaleBarStartX,
                    m_nGrayscaleStartY,
                    i + m_nGrayscaleBarStartX,
                    m_nGrayscaleStartY + m_nBarHeight);
            }
            m_nSampleHeight = m_nGrayscaleStartY - m_nSampleStartY - 6 - 2 - 2;
        }
        else
        {
            m_nRColorStartY = m_nHeight - 2 - (m_nBarHeight + 2 + 6) * 6;
            m_nGColorStartY = m_nRColorStartY + (m_nBarHeight + 2 + 6);
            m_nBColorStartY = m_nRColorStartY + (m_nBarHeight + 2 + 6) * 2;
            m_nHColorStartY = m_nRColorStartY + (m_nBarHeight + 2 + 6) * 3;
            m_nSColorStartY = m_nRColorStartY + (m_nBarHeight + 2 + 6) * 4;
            m_nVColorStartY = m_nRColorStartY + (m_nBarHeight + 2 + 6) * 5;
            m_nColorBarStartX = 2 + m_objFont.charWidth('H') + 2 + 1;
            m_nBarWidth = m_nTextStartX - m_nColorBarStartX - 4;
            g.setColor(m_nTextColor);
            g.drawString("R", 2, m_nRColorStartY, Graphics.LEFT | Graphics.TOP);
            g.drawString("G", 2, m_nGColorStartY, Graphics.LEFT | Graphics.TOP);
            g.drawString("B", 2, m_nBColorStartY, Graphics.LEFT | Graphics.TOP);
            g.drawString("H", 2, m_nHColorStartY, Graphics.LEFT | Graphics.TOP);
            g.drawString("S", 2, m_nSColorStartY, Graphics.LEFT | Graphics.TOP);
            g.drawString("V", 2, m_nVColorStartY, Graphics.LEFT | Graphics.TOP);
            m_nColorBarsCount = 6;
            m_nSampleHeight = m_nRColorStartY - m_nSampleStartY - 6 - 2 - 2;
        }
        m_nCurrentColorBar = 0;
        g.setColor(m_nBorderColor);
        g.drawRect(
            m_nOldSampleStartX - 1,
            m_nSampleStartY - 1,
            m_nSampleWidth * 2 + 1,
            m_nSampleHeight + 1);
    }
    
    public void setPreviousDisplayable(Displayable d)
    {
        m_objPreviousDisplayable = d;
    }
    
    private void updateGrayscale()
    {
        m_nColor = (m_nGrayscale << 16) + (m_nGrayscale << 8) + m_nGrayscale;
    }
    
    private void updateRGB()
    {
        m_nColor = Color.fromHSV(m_nHColor, m_nSColor, m_nVColor);
        m_nRColor = Color.red(m_nColor);
        m_nGColor = Color.green(m_nColor);
        m_nBColor = Color.blue(m_nColor);
    }
    
    private void updateHSV()
    {
        m_nColor = Color.fromRGB(m_nRColor, m_nGColor, m_nBColor);
        m_nHColor = Color.hue(m_nColor);
        m_nSColor = Color.saturation(m_nColor);
        m_nVColor = Color.value(m_nColor);
    }
}
