/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Interface;

import javax.microedition.lcdui.*;
import java.util.*;

public class ColorThemeKeeper
{
    private Display m_objDisplay;

    private Image   m_imgBackground;
    private int     m_nGoodMessageColor;
    private int     m_nBadMessageColor;
    private int     m_nBackgroundColor;
    private int     m_nBorderColor;
    private int     m_nTextColor;
    private int     m_nSelectionColor;
    private int     m_nGaugeColor;
    private int     m_nSideMessageColor;
    
    private static final int BACKGROUND_CHUNKS_COUNT    = 6;
    
    public int getBackgroundColor()
    {
        return m_nBackgroundColor;
    }
    
    public Image getBackgroundImage()
    {
        return m_imgBackground;
    }
    
    public int getBadMessageColor()
    {
        return m_nBadMessageColor;
    }
    
    public int getBorderColor()
    {
        return m_nBorderColor;
    }
    
    public int getGaugeColor()
    {
        return m_nGaugeColor;
    }
    
    public int getGoodMessageColor()
    {
        return m_nGoodMessageColor;
    }
    
    public int getSelectionColor()
    {
        return m_nSelectionColor;
    }
    
    public int getSideMessageColor()
    {
        return m_nSideMessageColor;
    }
    
    public int getTextColor()
    {
        return m_nTextColor;
    }
    
    public boolean isColor()
    {
        return m_objDisplay.isColor();
    }
    
    public void setDisplayObject(Display objDisplay)
    {
        m_objDisplay = objDisplay;
        updateTheme();
    }
    
    public void setSize(int nWidth,
                        int nHeight)
    {
        Random      objRandom;
        int         nRandom;
        int         nBaseLineColor;
        int         i, j;
        Graphics    g;
        Image       imgBackgroundChunk;
        int         nBackgroundChunkWidth;
        
        nBackgroundChunkWidth = nWidth / BACKGROUND_CHUNKS_COUNT;
        imgBackgroundChunk = Image.createImage(nBackgroundChunkWidth, nHeight);
        g = imgBackgroundChunk.getGraphics();
        objRandom = new Random();
        for (i = nHeight; --i >= 0;)
        {
            nBaseLineColor = 160 - 128 * i / nHeight;
            for (j = nWidth / BACKGROUND_CHUNKS_COUNT; --j >= 0;)
            {
                nRandom = objRandom.nextInt() / 3;
                nRandom %= 30;
                g.setGrayScale(nBaseLineColor + nRandom);
                g.drawLine(j, i, j, i);
            }
        }
        m_imgBackground = Image.createImage(nWidth, nHeight);
        g = m_imgBackground.getGraphics();
        for (i = 0; i < nWidth; i += nBackgroundChunkWidth)
            g.drawImage(imgBackgroundChunk, i, 0, Graphics.LEFT | Graphics.TOP);
    }
    
    private void updateTheme()
    {
        if (m_objDisplay.isColor())
        {
            m_nBorderColor      = 0xA00050;
            m_nBadMessageColor  = 0xFF0000;
            m_nGoodMessageColor = 0x00FF00;
            m_nGaugeColor       = 0x0050A0;
            m_nSideMessageColor = 0x5050FF;
        }
        else
        {
            m_nBorderColor      = 0xFFFFFF;
            m_nBadMessageColor  = 0xFFFFFF;
            m_nGoodMessageColor = 0xFFFFFF;
            m_nGaugeColor       = 0xA0A0A0;
            m_nSideMessageColor = 0xFFFFFF;
        }
        m_nSelectionColor   = 0xFFFFFF;
        m_nBackgroundColor  = 0x000000;
        m_nTextColor        = 0xFFFFFF;
    }
}