/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Interface;

import Game.*;
import Main.*;
import java.lang.*;
import javax.microedition.lcdui.*;

public class ScoresTableCanvas extends Canvas implements CommandListener, Localizable
{
    private Tetris              m_objApplication;
    private ScoresTable         m_objScoresTable;
    private ColorThemeKeeper    m_objColorThemeKeeper;
    private Localizer           m_objLocalizer;
    private Image               m_imgBackground;
    private Font                m_objFont;
    private int                 m_nFontLineHeight;
    private int                 m_nScrollPosition;
    private int                 m_nWidth;
    private int                 m_nHeight;
    private int                 m_nScreenLinesCount;
    private int                 m_nNamesStartX;
    private int                 m_nScrollBarStartY;
    private int                 m_nScrollBarStartX;
    private int                 m_nScrollBarWidth;
    private int                 m_nScrollBarHeight;
    private Command             m_cmdBack;
    private Command             m_cmdClear;
    
    private int                 m_nTextColor;
    private int                 m_nScrollBarColor;
    private int                 m_nBorderColor;
    private int                 m_nBackgroundColor;

    public ScoresTableCanvas(Tetris objApplication, Localizer objLocalizer)
    {
        m_objApplication = objApplication;
        m_objLocalizer = objLocalizer;
        
        m_objFont = Font.getFont(
            Font.FACE_PROPORTIONAL,
            Font.STYLE_BOLD,
            Font.SIZE_MEDIUM);
        m_nFontLineHeight = m_objFont.getHeight();
        m_nWidth = getWidth();
        m_nHeight = getHeight();
        m_nScreenLinesCount = m_nHeight / m_nFontLineHeight;
        m_nNamesStartX = 2 + m_objFont.charsWidth("30.".toCharArray(), 0, 3);
        
        m_nScrollBarStartY = 3;
        m_nScrollBarHeight = m_nHeight - 3 - 3;
        m_nScrollBarWidth = 3;
        m_nScrollBarStartX = m_nWidth - 3 - m_nScrollBarWidth;
        
        setCommandListener(this);
        localize();
    }
    
    public void commandAction(Command       cmd,
                              Displayable   d)
    {
        if (cmd == m_cmdClear)
        {
            m_objScoresTable.cleanTable();
            m_nScrollPosition = 0;
        }
        m_objApplication.enterMenu();
    }
    
    private void keyEvent(int nKeyCode)
    {
        boolean bRepaint = false;
        
        switch (nKeyCode)
        {
            case -1:    //  Up
            case KEY_NUM2:
            case UP:
                if (m_nScrollPosition > 0)
                {
                    m_nScrollPosition--;
                    bRepaint = true;
                }
                break;

            case -2:    //  Down
            case KEY_NUM8:
            case DOWN:
                if (m_nScrollPosition + m_nScreenLinesCount < m_objScoresTable.getRecordsCount())
                {
                    m_nScrollPosition++;
                    bRepaint = true;
                }
        }
        if (bRepaint)
            repaint();
    }
    
    protected void keyPressed(int nKeyCode)
    {
        keyEvent(nKeyCode);
    }
    
    protected void keyRepeated(int nKeyCode)
    {
        keyEvent(nKeyCode);
    }
    
    public void localize()
    {
        if (m_cmdBack != null)
        {
            removeCommand(m_cmdBack);
            removeCommand(m_cmdClear);
        }
        m_cmdBack = new Command(m_objLocalizer.get(Localizer.S_BACK), Command.BACK, 0);
        m_cmdClear = new Command(m_objLocalizer.get(Localizer.S_CLEAR), Command.SCREEN, 1);
        addCommand(m_cmdBack);
        addCommand(m_cmdClear);
    }
    
    protected void paint(Graphics g)
    {
        int i;
        int nLastRecord;
        int nRecordsCount = m_objScoresTable.getRecordsCount();
        int nCurrentRowStartY;
        int nSliderStartY;
        int nSliderHeight;
        
        nLastRecord = Math.min(m_nScrollPosition + m_nScreenLinesCount, nRecordsCount);

        g.drawImage(m_imgBackground, 0, 0, Graphics.LEFT | Graphics.TOP);
        g.setColor(m_nTextColor);
        for (i = m_nScrollPosition; i < nLastRecord; i++)
        {
            nCurrentRowStartY = (i - m_nScrollPosition) * m_nFontLineHeight;
            g.drawString(
                String.valueOf(i + 1),
                2,
                nCurrentRowStartY,
                Graphics.LEFT | Graphics.TOP);
            g.drawString(
                m_objScoresTable.getNameAtPosition(i + 1),
                m_nNamesStartX,
                nCurrentRowStartY,
                Graphics.LEFT | Graphics.TOP);
            g.drawString(
                String.valueOf(m_objScoresTable.getScoresAtPosition(i + 1)),
                m_nScrollBarStartX - 2,
                nCurrentRowStartY,
                Graphics.RIGHT | Graphics.TOP);
        }
        if (nRecordsCount == 0)
        {
            g.drawString(
                m_objLocalizer.get(Localizer.S_NO_RECORDS),
                m_nWidth / 2,
                m_nHeight / 2,
                Graphics.HCENTER | Graphics.TOP);
        }
        else
        {
            g.setColor(m_nBorderColor);
            g.drawRoundRect(
                m_nScrollBarStartX - 1,
                m_nScrollBarStartY - 1,
                m_nScrollBarWidth + 1,
                m_nScrollBarHeight + 1,
                2,
                2);
            g.setColor(m_nBackgroundColor);
            g.fillRect(
                m_nScrollBarStartX,
                m_nScrollBarStartY,
                m_nScrollBarWidth,
                m_nScrollBarHeight);
            g.setColor(m_nScrollBarColor);
            nSliderStartY =
                m_nScrollPosition * m_nScrollBarHeight / nRecordsCount +
                m_nScrollBarStartY;
            nSliderHeight =
                (nLastRecord - m_nScrollPosition) * m_nScrollBarHeight /
                nRecordsCount;
            g.fillRect(
                m_nScrollBarStartX,
                nSliderStartY,
                m_nScrollBarWidth,
                nSliderHeight);
        }
    }
    
    public void setColorThemeKeeperObject(ColorThemeKeeper objColorThemeKeeper)
    {
        m_objColorThemeKeeper = objColorThemeKeeper;
        
        m_imgBackground     = objColorThemeKeeper.getBackgroundImage();
        m_nTextColor        = objColorThemeKeeper.getTextColor();
        m_nBorderColor      = objColorThemeKeeper.getBorderColor();
        m_nScrollBarColor   = objColorThemeKeeper.getGaugeColor();
        m_nBackgroundColor  = objColorThemeKeeper.getBackgroundColor();
    }
    
    public void setScoresTableObject(ScoresTable objScoresTable)
    {
        m_objScoresTable = objScoresTable;
    }
}