/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Interface;

import Main.*;
import javax.microedition.lcdui.*;

public class SoundDialogue extends Form implements CommandListener, Localizable
{
    private Tetris          m_objApplication;
    private Localizer       m_objLocalizer;
    private ChoiceGroup     m_chkMute;
    private SettingsKeeper  m_objSettingsKeeper;
    private Gauge           m_objVolume;
    
    private boolean         m_arrMute[] = new boolean[1];
    private int             m_nVolume;
    
    private Command         m_cmdOK = new Command("OK", Command.OK, 0);
    private Command         m_cmdCancel;
    
    public SoundDialogue(Tetris         objApplication,
                         Localizer      objLocalizer,
                         SettingsKeeper objSettingsKeeper)
    {
        super("");

        m_arrMute[0] = objSettingsKeeper.getMute();
        m_nVolume = objSettingsKeeper.getVolume();

        m_objApplication = objApplication;
        m_objLocalizer = objLocalizer;
        m_objSettingsKeeper = objSettingsKeeper;
        m_chkMute = new ChoiceGroup("", Choice.MULTIPLE);
        m_objVolume = new Gauge("", true, 10, 0);

        append(m_chkMute);
        append(m_objVolume);
        
        addCommand(m_cmdOK);
        setCommandListener(this);
        
        localize();
    }
    
    public int hotKeyLouder()
    {
        if (m_nVolume <= 90)
            setVolume(m_nVolume + 10);
        return m_nVolume;
    }
    
    public boolean hotKeyMute()
    {
        setMute(!m_arrMute[0]);
        return m_arrMute[0];
    }
    
    public int hotKeyQuieter()
    {
        if (m_nVolume >= 10)
            setVolume(m_nVolume - 10);
        return m_nVolume;
    }
    
    private boolean isMuteChecked()
    {
        boolean arrMute[] = new boolean[1];
        
        m_chkMute.getSelectedFlags(arrMute);
        return arrMute[0];
    }
    
    public void localize()
    {
        setTitle(m_objLocalizer.get(Localizer.S_SOUND));
        
        if (m_cmdCancel == null)
            removeCommand(m_cmdCancel);
        m_cmdCancel = new Command(m_objLocalizer.get(Localizer.S_CANCEL), Command.CANCEL, 1);
        addCommand(m_cmdCancel);
        
        m_chkMute.deleteAll();
        m_chkMute.append(m_objLocalizer.get(Localizer.S_MUTE), null);
        m_objVolume.setLabel(m_objLocalizer.get(Localizer.S_VOLUME));
        readSettings();
    }
    
    public void commandAction(Command cmd, Displayable d)
    {
        if (cmd == m_cmdOK)
        {
            setVolume(m_objVolume.getValue() * 10);
            setMute(isMuteChecked());
        }
        m_objApplication.onBack();
    }
    
    public void readSettings()
    {
        m_objVolume.setValue(m_nVolume / 10);
        m_chkMute.setSelectedFlags(m_arrMute);
    }
    
    private void setMute(boolean bMute)
    {
        m_objApplication.setMute(bMute);
        m_objSettingsKeeper.setMute(bMute);
        m_arrMute[0] = bMute;
    }
    
    private void setVolume(int nVolume)
    {
        m_objApplication.setVolume(nVolume);
        m_objSettingsKeeper.setVolume(nVolume);
        m_nVolume = nVolume;
    }
}